/***********************************************************************
 *
 * TITLE:	colors.c
 *
 * AUTHOR:	Cassie Mulnix
 *
 * DESCRIPTION:	This module is part of the OPPS editor: xopps.  It contains
 *		routines to handle color.
 *
 *                              CHANGE HISTORY
 *
 * $Log: color.c,v $
 * Revision 1.9  1994/06/01  22:26:22  clm
 * changed color from structure to 3 arrays
 *
 * Revision 1.8  1994/05/20  23:31:43  clm
 * updated with HP, SGI changes
 *
 * Revision 1.7  1994/05/04  23:34:36  clm
 * ported to ANSI C
 *
 * Revision 1.6  1992/08/04  17:01:10  clm
 * changed malloc and free to XtMalloc and XtFree
 *
 * Revision 1.5  1992/05/20  22:28:51  clm
 * *** empty log message ***
 *
 * Revision 1.4  1992/05/20  19:50:57  clm
 * cleaned up file and help format
 *
 * Revision 1.3  1992/05/11  22:38:24  kevin
 * renamed project include file to xopps.h
 *
 * Revision 1.1  1990/12/19  08:55:18  ana
 * Initial revision
 *
 ***********************************************************************
 *
 * WARNINGS:
 *
 * EXTERNAL CALLABLE COMPONENTS (PUBLIC):
 *
 * GLOBALS:
 *
 * WAIVERS:
 *
 * NOTES:
 *
 * MANPAGE:
 *
 ***********************************************************************/

#ifndef lint
static char rcsid[] =
    "$Id: color.c,v 1.9 1994/06/01 22:26:22 clm OEL $";
#endif

#include "xopps.h"

static void add_a_color(int index, int gscale, char *name);

/***********************************************************************
 *
 * FUNCTION:	load_colors()
 *
 * INPUTS:	buf	(char *)
 *
 * OUTPUTS:
 *
 * RETURNS:
 *
 * EXTERNALLY READ:
 *
 * EXTERNALLY MODIFIED:
 *
 * DESCRIPTION:	loads the colors into the buffer
 */

int 
load_colors(char *buf)
{
    int i, gscale;
    char name[50];

    if (record_scan(buf + 1, "dds", &i, &gscale, name) == 3) {
	add_a_color(i+2, gscale, name);	/* add 2 to index cuz black and white 
				   	 are already added */
        return 0;
    } else {
        return -1;
    }
}

/***********************************************************************
 *
 * FUNCTION:	init_colors()
 *
 * INPUTS:
 *
 * OUTPUTS:
 *
 * RETURNS:
 *
 * EXTERNALLY READ:
 *
 * EXTERNALLY MODIFIED:
 *
 * DESCRIPTION:	initializes the colors
 */

void 
init_colors(void)
{
    register_io_obj('K', load_colors);

    add_a_color(0, 100, "black");
    add_a_color(1, 100, "white");
    add_a_color(2, 100, "blue");
    add_a_color(3, 100, "green");
    add_a_color(4, 100, "red");
    add_a_color(5, 100, "yellow");
}

/***********************************************************************
 *
 * FUNCTION:	add_a_color()
 *
 * INPUTS:	index, gscale	(int)
 *		name		(char *)
 *
 * OUTPUTS:
 *
 * RETURNS:
 *
 * EXTERNALLY READ:
 *
 * EXTERNALLY MODIFIED:
 *
 * DESCRIPTION:	adds a color to the color array
 */

static void add_a_color(int index, int gscale, char *name)
{

    color_index[ncolor] = index;
    color_gscale[ncolor] = gscale;
    color_name[ncolor] = XtNewString(name);
    ++ncolor;
}
