/***********************************************************************
 *
 * TITLE:
 *	dialog.h
 *
 * AUTHOR:
 *	Kevin J. Miller
 *
 * DESCRIPTION:
 *	Common header file for dialog.c functions.
 *
 *				CHANGE HISTORY
 *
 * $Log: dialog.h,v $
 * Revision 1.11  1994/09/09  22:06:48  chester
 * Add DialogList routines.
 *
 * Revision 1.10  1994/06/21  00:44:09  kevin
 * added Toggle functions
 *
 * Revision 1.9  1994/06/21  00:34:15  kevin
 * added Duration functions
 *
 * Revision 1.8  1994/06/17  03:51:17  kevin
 * added optional prototypes
 *
 * Revision 1.7  1994/06/16  03:04:12  kevin
 * added: second size on DialogTextCreate
 * added: DialogLabelCreate, DialogButtonCreate, DialogSeparatorCreate,
 *        DialogSpecialCreate, DialogChangeLabel
 *
 * Revision 1.6  1994/06/14  22:32:59  kevin
 * added int and float function to Text
 *
 * Revision 1.5  1994/06/10  14:25:35  ana
 * added a label widget to DialogText structure
 *
 * Revision 1.4  1994/06/06  23:26:12  clm
 * added RadioBox options
 *
 * Revision 1.3  1994/05/26  21:32:23  kevin
 * added array and Pixmaps to Option Menu
 *
 * Revision 1.2  1994/05/26  21:08:52  kevin
 * added date functions
 *
 * Revision 1.1  1994/05/18  18:17:43  kevin
 * Initial revision
 *
 ***********************************************************************
 *
 * WARNINGS:
 *
 * EXTERNAL CALLABLE COMPONENTS (PUBLIC):
 *
 * GLOBALS:
 *
 * WAIVERS:
 *
 * NOTES:
 *
 * MANPAGE:
 *
 ***********************************************************************/

#ifndef dialog_h
#define dialog_h

#ifndef lint
static char rcsid_dialog[] = 
    "$Id: dialog.h,v 1.11 1994/09/09 22:06:48 chester OEL $";
#endif

/*
 * global type definitions
 */
typedef struct {
    Widget parent;
    Widget label;
} DialogEntry;

typedef struct {
    Widget parent;
    Widget label;
    Widget text;
    int maxcols;
} DialogText;

typedef struct {
    Widget parent;
    Widget label;
    Widget list;
} DialogList;

typedef struct {
    Widget parent;
    Widget label;
    Widget menu;
    int num;
    Widget *button;
    int *value;
} DialogOption;

typedef struct {
    Widget parent;
    Widget label;
    Widget year;
    Widget day;
    Widget hour;
    Widget minute;
    Widget second;
} DialogTime;

typedef struct {
    Widget parent;
    Widget label;
    Widget month;
    Widget day;
    Widget year;
} DialogDate;

typedef struct {
    Widget parent;
    Widget label;
    int num;
    Widget *button;
    int *value;
} DialogRadio;

typedef struct {
    Widget parent;
    Widget label;
    Widget day;
    Widget hour;
    Widget minute;
    Widget second;
} DialogDuration;

typedef struct {
    Widget parent;
    Widget label;
    int num;
    Widget * button;
    int * keys;
} DialogToggle;

/*
 * global constant definitions
 */
#define DIALOG_NO_CHANGE	0
#define DIALOG_UPDATE		1
#define DIALOG_ERROR		2
#define DIALOG_CANCEL		4

/*
 * dialog option types
 */
enum DialogOptionType {
    DIALOG_OPT_STR_VA, DIALOG_OPT_STR_ARY, DIALOG_OPT_PIX_VA,
    DIALOG_OPT_PIX_ARY
};

enum DialogRadioType {
    DIALOG_RAD_STR_VA, DIALOG_RAD_STR_ARY
};

enum DialogToggleType  {
    DIALOG_TOG_STR_VA, DIALOG_TOG_STR_ARY
};

#ifdef _NO_PROTO
#define P_(x) ()
#else
#define P_(x) x
#endif

/*
 * global function prototypes
 */
extern Widget DialogInit P_((Widget, char *, char *, XtCallbackProc,
    XtPointer));
extern void DialogAddRow P_((int));
extern void DialogNextWidth P_((int));
extern DialogText *DialogTextCreate P_((char *, int, int));
extern void DialogTextSet P_((DialogText *, char *));
extern char *DialogTextGet P_((DialogText *));
extern void DialogTextSetInt P_((DialogText *, int));
extern int DialogTextGetInt P_((DialogText *));
extern void DialogTextSetDouble P_((DialogText *, double));
extern double DialogTextGetDouble P_((DialogText *));
extern DialogList *DialogListCreate P_((char *, int, unsigned char,
    XtCallbackProc));
extern void DialogListSetItems(DialogList *, char **);
extern void DialogListSelectItems(DialogList *, char **);
extern void DialogListDeselectItems(DialogList *);
extern void DialogListGetItems P_((DialogList *, char ***, int *));
extern void DialogListGetSelectItems P_((DialogList *, char ***, int *));
extern void DialogListAddItem P_((DialogList *, char *));
extern void DialogListDeleteItem P_((DialogList *, char *));
extern void DialogListDeleteList P_((DialogList *));
extern DialogOption *DialogOptionCreate P_((char *, int, int, XtCallbackProc,
    enum DialogOptionType, ...));
extern void DialogOptionSet P_((DialogOption *, int));
extern int DialogOptionGet P_((DialogOption *));
extern DialogRadio *DialogRadioCreate P_((char *, int, int, XtCallbackProc,
    enum DialogRadioType, ...));
extern void DialogRadioSet P_((DialogRadio *, int));
extern int DialogRadioGet P_((DialogRadio *));
extern DialogToggle *DialogToggleCreate P_((char *, int, XtCallbackProc,
    enum DialogToggleType, ...));
extern void DialogToggleSet P_((DialogToggle *, int, int));
extern int DialogToggleGet P_((DialogToggle *, int));
extern void DialogToggleFlip P_((DialogToggle *, int));
extern void DialogToggleDestroy P_((DialogToggle *));
extern DialogTime *DialogTimeCreate P_((char *, int));
extern void DialogTimeSet P_((DialogTime *, utc_val));
extern utc_val DialogTimeGet P_((DialogTime *));
extern DialogDate *DialogDateCreate P_((char *));
extern void DialogDateSet P_((DialogDate *, utc_val));
extern utc_val DialogDateGet P_((DialogDate *));
extern DialogDuration *DialogDurationCreate(char *, int, int);
extern void DialogDurationSet(DialogDuration *, int);
extern int DialogDurationGet(DialogDuration *);
extern DialogEntry *DialogLabelCreate P_((char *));
extern DialogEntry *DialogButtonCreate P_((char *, XtCallbackProc, XtPointer));
extern DialogEntry *DialogSeparatorCreate P_((int));
extern DialogEntry *DialogSpecialCreate P_((char *));
extern void DialogStdButtons P_((XtCallbackProc, XtPointer, XtCallbackProc,
    XtPointer, XtCallbackProc, XtPointer));
extern int DialogManage P_((Widget));
extern void DialogFlagSet P_((int));
extern void DialogChangeLabel P_((DialogEntry *, char *));

#endif
