/***********************************************************************
 *
 * TITLE:       mover.c
 *
 * AUTHOR:      Cassie Mulnix
 *
 * DESCRIPTION: This module is part of the OPPS editor, xopps.  It contains
 *              generic routines to move objects on the drawing area.
 *
 *                              CHANGE HISTORY
 *
 * $Log: mover.c,v $
 * Revision 1.26  1994/05/20  23:31:43  clm
 * updated with HP, SGI changes
 *
 * Revision 1.25  1994/05/04  23:34:36  clm
 * ported to ANSI C
 *
 * Revision 1.24  1992/12/04  17:57:54  clm
 * changed MAX_PLN to npln
 *
 * Revision 1.23  1992/11/09  21:47:44  clm
 * removed unused variable
 *
 * Revision 1.22  1992/11/09  16:30:26  clm
 * added precedence lines
 *
 * Revision 1.21  1992/10/16  18:09:17  clm
 * fixed expose for finish_H_move
 *
 * Revision 1.20  1992/10/02  18:43:11  clm
 * fixed finish move for LINE_Y
 *
 * Revision 1.19  1992/08/25  15:54:06  clm
 * incorporated xlnary and ylnary objects into move object routines
 *
 * Revision 1.18  1992/07/22  22:06:05  clm
 * changed mov to send offset only
 *
 * Revision 1.17  1992/07/15  19:43:18  clm
 * changed current_date to issue_date
 *
 * Revision 1.16  1992/07/08  17:21:15  clm
 * changed issue_date to current_date call
 *
 * Revision 1.15  1992/07/06  17:49:54  clm
 * fixed comment format
 * got H_mover functions workingt
 *
 * Revision 1.14  1992/06/24  17:09:16  clm
 * added horizontal line routines
 *
 * Revision 1.12  1992/05/26  15:49:53  clm
 * updated mover routine
 *
 * Revision 1.8  1991/08/10  02:07:38  kevin
 * added multiple move rectangles
 *
 * Revision 1.7  1990/12/28  14:28:01  kevin
 * added highlight GC
 *
 * Revision 1.6  90/12/19  15:26:59  kevin
 * moved issue_date() before add_expose_event(0, ...) because it may contain
 * an expose event
 *
 * Revision 1.5  90/12/19  14:28:54  kevin
 * set refresh to depend on objects using loc_ function
 *
 * Revision 1.4  90/12/13  18:42:56  kevin
 * fixed refresh
 *
 * Revision 1.3  90/12/11  15:37:27  ana
 * Incorporated the undo feature.
 *
 * Revision 1.2  90/12/10  18:46:46  kevin
 * got basic move function working
 * 
 * Revision 1.1  90/12/07  18:44:06  kevin
 * Initial revision
 * 
 */

/***********************************************************************
 *
 * WARNINGS:
 *
 * EXTERNAL CALLABLE COMPONENTS (PUBLIC):
 *
 * GLOBALS:
 *
 * WAIVERS:
 *
 * NOTES:
 *
 * MANPAGE:
 *
 ***********************************************************************/

#ifndef lint
static char rcsid[] =
    "$Id: mover.c,v 1.26 1994/05/20 23:31:43 clm OEL $";
#endif

#include "xopps.h"


static int co;			/* current object */
static int x_start, y_start;	/* original coordinates */
static int x_off, y_off;	/* current offset of object */
static int x_min, y_min;	/* minimum permitted location */
static int x_max, y_max;	/* maximum permitted location */
static XRectangle *(*r)[];		/* array of moving rectangles */

static void expose_object(int);
static void mover(Widget, XtPointer, XEvent *);	/* object move handler */
static void H_mover(Widget, XtPointer, XEvent *);	/* Hobject move handler */

/***********************************************************************
 *
 * FUNCTION:
 *	begin_move()
 *
 * INPUTS:
 *	w		(Widget)
 *	co_p		(int)
 *	x, y		(int)
 *
 * OUTPUTS:
 *
 * RETURNS:
 *
 * EXTERNALLY READ:
 *
 * EXTERNALLY MODIFIED:
 *
 * DESCRIPTION:
 *	called when the middle mouse button is pressed on an object
 */

void 
begin_move(Widget w, int co_p, int x, int y, int mov_type)
{
    int i;			/* temp index */

    co = co_p;
    switch (mov_type) {
    case OBJECT:
        r = getMoveRect(objary[co].rlist);
        (*(objary[co].opsptr->lim))(co, &x_min, &y_min, &x_max, &y_max, OBJECT);
	break;
    case LINE_X:
        r = getMoveRect(xlnary[co].rlist);
        (*(xlnary[co].opsptr->lim))(co, &x_min, &y_min, &x_max, &y_max, LINE_X);
	break;
    case LINE_Y:
        r = getMoveRect(ylnary[co].rlist);
        (*(ylnary[co].opsptr->lim))(co, &x_min, &y_min, &x_max, &y_max, LINE_Y);
	break;
    }
    x_start = x;
    y_start = y;
    x_off = 0;
    y_off = 0;
    x_min -= (*r)[0]->x;
    y_min -= (*r)[0]->y;
    x_max -= (*r)[0]->x;
    y_max -= (*r)[0]->y;
    for (i = 0; (*r)[i] != NULL; ++i) {
	XDrawRectangle(dw.display, dw.win, hilite_gc, (*r)[i]->x + x_off,
	    (*r)[i]->y + y_off, (unsigned)((*r)[i]->width), 
	    (unsigned)((*r)[i]->height));
    }
    XtAddEventHandler(w, Button2MotionMask, FALSE, (XtEventHandler)mover, NULL);
}

/***********************************************************************
 *
 * FUNCTION:
 *	begin_H_move()	
 *
 * INPUTS:
 *	w  	(Widget)
 *      co_p	(int) 
 *     	x, y	(int) 
 *
 * OUTPUTS:
 *
 * RETURNS:
 *
 * EXTERNALLY READ:
 *
 * EXTERNALLY MODIFIED:
 *
 * DESCRIPTION:
 *	begins the move of the selected horizontal line	
 */

void 
begin_H_move(Widget w, int co_p, int x, int y)
{
    int i;			/* temp index */

    co = co_p;
    r = getMoveRect(Hobjary[co].rlist);
    x_start = x;
    y_start = y;
    x_off = 0;
    y_off = 0;
    (*(Hobjary[co].opsptr->lim))(co, &x_min, &y_min, &x_max, &y_max, OBJECT);
    x_min -= (*r)[0]->x;
    y_min -= (*r)[0]->y;
    x_max -= (*r)[0]->x;
    y_max -= (*r)[0]->y;
    for (i = 0; (*r)[i] != NULL; ++i) {
	XDrawRectangle(dw.display, dw.win, hilite_gc, x_left_margin,
	    (*r)[i]->y + y_off, (unsigned)((*r)[i]->width), 
	    (unsigned)((*r)[i]->height));
    }
    XtAddEventHandler(w, Button2MotionMask, FALSE, (XtEventHandler)H_mover, 
	NULL);
}

/***********************************************************************
 *
 * FUNCTION:
 *	finish_move()
 *
 * INPUTS:
 *	w  	(Widget)
 *
 * OUTPUTS:
 *
 * RETURNS:
 *
 * EXTERNALLY READ:
 *
 * EXTERNALLY MODIFIED:
 *
 * DESCRIPTION:
 *	finishes the move when the mouse button is released 
 */

void 
finish_move(Widget w, int mov_type)
{
    int i;

    XtRemoveEventHandler(w, Button2MotionMask, FALSE, (XtEventHandler)mover, 
	NULL);
    for (i = 0; (*r)[i] != NULL; ++i) {
	XDrawRectangle(dw.display, dw.win, hilite_gc, (*r)[i]->x + x_off,
	    (*r)[i]->y + y_off, (unsigned)((*r)[i]->width), 
	    (unsigned)((*r)[i]->height));
    }
    if (x_off || y_off) {		/* did move */
	store_object(co, BACK_MOVE, mov_type);
	switch(mov_type) {
	case OBJECT:
	    exposeRectList(1, objary[co].rlist);
	    (*(objary[co].opsptr->mov))(co, x_off, y_off, OBJECT);
	    (*(objary[co].opsptr->loc))(co, OBJECT);
	    for (i = 0; i < npln; i++) {
	  	loc_pline(i, OBJECT);
	    }
	    issue_date();
	    expose_object(co);
	    break;
	case LINE_X:
	    exposeRectList(1, xlnary[co].rlist);
	    (*(xlnary[co].opsptr->mov))(co, x_off, y_off, LINE_X);
	    (*(xlnary[co].opsptr->loc))(co, LINE_X);
	    issue_date();
	    exposeRectList(0, xlnary[co].rlist);
	    /*
	     * relocate all other objects
	     */
	    for (i = 0; i < nyln; i++) {
		(*(ylnary[i].opsptr->loc))(i, LINE_Y);
	    }
	    set_dsp_size();
	    for (i = 0; i < Hnobj; i++) {
		(*(Hobjary[i].opsptr->loc))(i, OBJECT);
		(*(Hobjary[i].opsptr->dsp))(i);
	    }
	    for (i = 0; i < nobj; i++) {
		(*(objary[i].opsptr->loc))(i, OBJECT);
		(*(objary[i].opsptr->dsp))(i);
	    }
	    for (i = 0; i < npln; i++) {
	  	loc_pline(i, OBJECT);
	    }
	    exposeRectList(0, objary[0].rlist);
	    break;
	case LINE_Y:
	    exposeRectList(1, ylnary[co].rlist);
	    (*(ylnary[co].opsptr->mov))(co, x_off, y_off, LINE_Y);
	    set_dsp_size();
	    (*(ylnary[co].opsptr->loc))(co, LINE_Y);
	    issue_date();
	    exposeRectList(0, ylnary[co].rlist);
	    /*
	     * relocate all other objects
	     */
	    for (i = 0; i < nxln; i++) {
		(*(xlnary[i].opsptr->loc))(i, LINE_X);
	    }
	    for (i = 0; i < Hnobj; i++) {
		(*(Hobjary[i].opsptr->dsp))(i);
	    }
	    for (i = 0; i < nobj; i++) {
		(*(objary[i].opsptr->loc))(i, OBJECT);
		(*(objary[i].opsptr->dsp))(i);
	    }
	    for (i = 0; i < npln; i++) {
	  	loc_pline(i, OBJECT);
	    }
	    exposeRectList(0, objary[0].rlist);
	    break;
	}
    }
}

/***********************************************************************
 *
 * FUNCTION:
 *	finish_H_move()	
 *
 * INPUTS:
 *	w	(Widget)	
 *
 * OUTPUTS:
 *
 * RETURNS:
 *
 * EXTERNALLY READ:
 *
 * EXTERNALLY MODIFIED:
 *
 * DESCRIPTION:
 *	finishes the horizontal line move when the mouse button is released
 */

void 
finish_H_move(Widget w)
{
    int i;			/* temp indices */
    XRectangle rect;		/* expose rectangle */

    XtRemoveEventHandler(w, Button2MotionMask, FALSE, (XtEventHandler)H_mover, 
	NULL);
    for (i = 0; (*r)[i] != NULL; ++i) {
        XDrawRectangle(dw.display, dw.win, hilite_gc, x_left_margin,
            (*r)[i]->y + y_off, (unsigned)((*r)[i]->width), 
	    (unsigned)((*r)[i]->height));
    }
    if (x_off || y_off) {               /* did move */
        exposeRectList(1, Hobjary[co].rlist);
        (*(Hobjary[co].opsptr->mov))(co, (*r)[0]->x+x_off, (*r)[0]->y + y_off,
	    OBJECT);
	issue_date();
	rect.x = 0;
   	rect.y = 0;
	rect.width = page_width;
	rect.height = page_height;
	AddExposeEvent(0, &rect);
    }
}

/***********************************************************************
 *
 * FUNCTION:
 *	mover()
 *
 * INPUTS:
 *	w		(Widget)
 *	client_data	(XtPointer)
 *	event		(XEvent)
 *
 * OUTPUTS:
 *
 * RETURNS:
 *
 * EXTERNALLY READ:
 *
 * EXTERNALLY MODIFIED:
 *
 * DESCRIPTION:
 *	moves the object while the middle mouse button is depressed
 */

static void 
mover(Widget w, XtPointer client_data, XEvent *event)
{
    int i;				/* temp index */

    for (i = 0; (*r)[i] != NULL; ++i) {
	XDrawRectangle(dw.display, dw.win, hilite_gc, (*r)[i]->x + x_off,
	    (*r)[i]->y + y_off, (unsigned)((*r)[i]->width), 
	    (unsigned)((*r)[i]->height));
    }

    x_off = event->xmotion.x - x_start;
    y_off = event->xmotion.y - y_start;

    /*
     * limit range of motion
     */
    if (x_off < x_min) {
 	x_off = x_min;
    } else if (x_off > x_max) {
 	x_off = x_max;
    }
    if (y_off < y_min) {
	y_off = y_min;
    } else if (y_off > y_max) {
	y_off = y_max;
    }

    for (i = 0; (*r)[i] != NULL; ++i) {
	XDrawRectangle(dw.display, dw.win, hilite_gc, (*r)[i]->x + x_off,
	    (*r)[i]->y + y_off, (unsigned)((*r)[i]->width), 
	    (unsigned)((*r)[i]->height));
    }
}

/***********************************************************************
 *
 * FUNCTION:
 *	H_mover()
 *
 * INPUTS:
 *	w		(Widget)
 *      client_data	(XtPointer)
 *	event		(XEvent)
 *
 * OUTPUTS:
 *
 * RETURNS:
 *
 * EXTERNALLY READ:
 *
 * EXTERNALLY MODIFIED:
 *
 * DESCRIPTION:
 *	tracks the horizontal line move while the mouse button is pressed
 */

static void 
H_mover(Widget w, XtPointer client_data, XEvent *event)
{
    int i;					/* temp index */

    for (i = 0; (*r)[i] != NULL; ++i) {
        XDrawRectangle(dw.display, dw.win, hilite_gc, x_left_margin,
	    (*r)[i]->y + y_off, (unsigned)((*r)[i]->width), 
	    (unsigned)((*r)[i]->height));
    }

    x_off = event->xmotion.x - x_start;
    y_off = event->xmotion.y - y_start;

    /*
     * limit range of motion
     */
    if (x_off < x_min) {
	x_off = x_min;
    } else if (x_off > x_max) {
	x_off = x_max;
    }
    if (y_off < y_min) {
	y_off = y_min;
    } else if (y_off > y_max) {
	y_off = y_max;
    }

    for (i = 0; (*r)[i] != NULL; ++i) {
        XDrawRectangle(dw.display, dw.win, hilite_gc, x_left_margin,
            (*r)[i]->y + y_off, (unsigned)((*r)[i]->width), 
	    (unsigned)((*r)[i]->height));
    }
}

static void 
expose_object(int co)
{
    switch (objary[co].type) {
    case OBJ_ACT:
    case OBJ_EVT:
    case OBJ_SACT:
    case OBJ_SEQ:
    case OBJ_SLP:
	exposeRectList(0, objary[0].rlist);
	break;
    default:
	exposeRectList(0, objary[co].rlist);
	break;
    }
}
