/***********************************************************************
 *
 * TITLE:	object.c
 *
 * AUTHOR:	Cassie Mulnix
 *
 * DESCRIPTION:	This module is part of the OPPS editor, xopps.  It contains
 *		common object management routines.
 *
 *                              CHANGE HISTORY
 *
 * $Log: object.c,v $
 * Revision 1.72  1994/08/10  22:08:02  cassie
 * removed init_pict and init_windows functions
 *
 * Revision 1.71  1994/06/08  18:46:20  clm
 * added sav function to typedefs and nul_fun assignments
 *
 * Revision 1.70  1994/06/02  22:25:57  clm
 * added MAC Pict and Windows META files options
 *
 * Revision 1.69  1994/06/02  19:44:09  clm
 * removed auto_areset and auto_ereset from init_objects since then
 * are now saved
 *
 * Revision 1.68  1994/06/01  22:30:21  clm
 * changed edit_object to EditObject to work with dialogs
 *
 * Revision 1.67  1994/05/04  23:34:36  clm
 * ported to ANSI C
 *
 * Revision 1.66  1993/05/27  16:49:09  clm
 * changed xopps_BACKUP to BACKUP.xopps
 *
 * Revision 1.65  1993/05/18  20:58:30  clm
 * added new message.c function
 *
 * Revision 1.64  1993/02/08  16:12:15  clm
 * changed MAX_PLN npln
 * fixed cancel for edit object if object is OBJ_STT
 *
 * Revision 1.63  1992/12/17  18:22:04  clm
 *  fixed new_pline so it won't exit when pline array is filled
 *
 * Revision 1.62  1992/12/11  21:31:15  clm
 * made RSRC_FONT and RSRC_POINT global
 *
 * Revision 1.61  1992/12/11  18:43:38  clm
 * fixed edit_H_object if cancel is pressed
 *
 * Revision 1.60  1992/12/10  22:46:03  clm
 * added font and point as resources for init_chart
 *
 * Revision 1.59  1992/12/10  21:45:40  clm
 * fixed new_index so chart is always object 0
 *
 * Revision 1.58  1992/12/04  17:58:09  clm
 * changed MAX_PLN to npln
 *
 * Revision 1.57  1992/12/02  19:24:45  clm
 * extra error checking to find_pline
 *
 * Revision 1.56  1992/11/12  18:59:12  clm
 * changed PERIOD to 5
 *
 * Revision 1.55  1992/11/10  15:52:50  clm
 * added dsp_pline if existing object is edited
 *
 * Revision 1.54  1992/11/09  21:48:17  clm
 * removed unused variable
 *
 * Revision 1.53  1992/11/09  16:31:45  clm
 * added precedence lines, new_pline routine
 *
 * Revision 1.52  1992/10/16  14:47:12  clm
 * changed free, realloc, and malloc to Xt functions
 *
 * Revision 1.51  1992/10/12  20:31:13  clm
 * added initialization of all_id_disp toggle
 *
 * Revision 1.50  1992/10/02  20:26:30  clm
 * housekeeping
 *
 * Revision 1.49  1992/10/02  18:43:39  clm
 * added y_gap and y_lines_end calculation after line_init call
 *
 * Revision 1.48  1992/09/24  15:44:26  clm
 * added save of auto_renum toggle
 *
 * Revision 1.47  1992/09/22  17:19:49  clm
 * added activity object
 *
 * Revision 1.46  1992/09/10  17:13:53  clm
 * added line_init call
 * removing xopps_init
 *
 * Revision 1.45  1992/09/08  22:50:28  clm
 * added init call for sequence object
 *
 * Revision 1.44  1992/08/31  17:02:44  clm
 * locate objects if chart is edited
 *
 * Revision 1.43  1992/08/27  16:14:17  clm
 * initialized vgrid option
 *
 * Revision 1.42  1992/08/26  17:18:43  clm
 * added toggle values to clear_all_obj
 *
 * Revision 1.41  1992/08/26  15:05:37  clm
 * added init_internals call to init_objects
 *
 * Revision 1.40  1992/08/25  20:08:15  clm
 * added boolean values for modified for beware message
 *
 * Revision 1.39  1992/08/25  16:24:26  clm
 * moved location of xopps_init call in init_objects
 *
 * Revision 1.38  1992/08/25  15:55:18  clm
 * added xlnary and ylnary objects to init
 *
 * Revision 1.37  1992/08/20  18:06:19  clm
 * added check_radiobox, set_radiobox, and check_titletext
 *
 * Revision 1.36  1992/08/20  16:12:01  clm
 * fixed store_object for handling status date object
 *
 * Revision 1.35  1992/08/04  17:01:10  clm
 * changed malloc and free to XtMalloc and XtFree
 *
 * Revision 1.34  1992/08/03  21:34:52  clm
 * added slip, status, page, chart date, symbol,and level init files
 *
 * Revision 1.33  1992/07/15  19:43:55  clm
 * changed current_date to issue_date
 *
 * Revision 1.32  1992/07/13  20:48:16  clm
 * removed clock and box objects
 * change color of hline if chart changes
 *
 * Revision 1.31  1992/07/08  17:25:50  clm
 * changed issue_date to current_date call
 *
 * Revision 1.30  1992/07/06  17:56:32  clm
 * fixed comment format
 *
 * Revision 1.29  1992/06/24  17:15:26  clm
 * added horizontal line functions
 * added check_year function
 *
 * Revision 1.28  1992/06/12  18:16:00  clm
 * made chart an object
 * add check_day module
 *
 * Revision 1.27  1992/05/28  22:15:30  clm
 * added check_year function
 *
 * Revision 1.24  1992/05/21  16:56:58  clm
 * changed to rectangle standars
 *
 * Revision 1.22  1992/05/20  19:50:57  clm
 * cleaned up file and help format
 *
 * Revision 1.21  1992/05/11  22:39:29  kevin
 * renamed project include file to xopps.h
 *
 * Revision 1.2  90/11/29  15:32:30  ana
 * First cut with motif widgets.
 * 
 * Revision 1.1  90/11/02  08:41:08  ana
 * Initial revision
 *
 ***********************************************************************
 *
 * WARNINGS:
 *
 * EXTERNAL CALLABLE COMPONENTS (PUBLIC):
 *
 * GLOBALS:
 *
 * WAIVERS:
 *
 * NOTES:
 *
 * MANPAGE:
 *
 ***********************************************************************/

#ifndef lint
static char rcsid[] =
    "$Id: object.c,v 1.72 1994/08/10 22:08:02 cassie OEL $";
#endif

/***************************************************************************
        new_index() - return index for new object
        clear_all_obj() - remove all objects
        find_object(x, y) - return index from location
        init_objects(parent) - initialize objects 
        rem_object(objptr) - set object to type OBJ_NUL
****************************************************************************/
#include "xopps.h"
#include <Xm/BulletinB.h>
#include <Xm/PushB.h>
#include <Xm/List.h>
#include <Xm/Text.h>
#include <Xm/ToggleB.h>

#define PERIOD 5		/* period for backups */

/*
 * typedefs for calling nul_func
 */
typedef void (*edt_fun)(int, int);
typedef void (*rem_fun)(int, int);
typedef void (*dsp_fun)(int);
typedef void (*lim_fun)(int, int *, int *, int *, int *, int);
typedef void (*loc_fun)(int, int);
typedef void (*mov_fun)(int, int, int, int);
typedef void (*cpy_fun)(struct obj *, struct obj *);
typedef void (*msg_fun)(int, int);
typedef void (*sav_fun)(int, FILE *, int);

static int bck_count = 0;	/* count to do periodic file backups */
static int lobj;		/* use for overlap search */
static XtAppContext app;	/* application context */

static void nul_func(void);

/***********************************************************************
 *
 * FUNCTION:
 *	nul_func()
 *
 * INPUTS:
 *	i	(int)
 *
 * OUTPUTS:
 *
 * RETURNS:
 *
 * EXTERNALLY READ:
 *
 * EXTERNALLY MODIFIED:
 *
 * DESCRIPTION:
 *	do nothing for OBJ_NUL
 */

static void 
nul_func(void)
{
    return;
}

/***********************************************************************
 *
 * FUNCTION:
 *	new_index()
 *
 * INPUTS:
 *	chart			(int) TRUE if object is a chart
 *
 * OUTPUTS:
 *
 * RETURNS:
 *
 * EXTERNALLY READ:
 *
 * EXTERNALLY MODIFIED:
 *
 * DESCRIPTION:
 *	returns an index for new object
 */

int 
new_index(int chart)
{
    int i;

    /*
     * want to insure that chart is always object 0
     */
    if (chart) {
	i = 0;
    } else {
        for (i = 1; i < nobj; i++) {
            if (objary[i].type == OBJ_NUL) break;
        }
    }
    if (i == nobj) {
        nobj++;
        if (nobj > MAX_OBJ) {
            fprintf(stderr, "Object array is filled\n");
            exit(1);
        }
    }
    objary[i].rlist = newRectList(0);
    lobj = 0;
    return i;
}

/***********************************************************************
 *
 * FUNCTION:
 *	new_H_index()
 *
 * INPUTS:
 *
 * OUTPUTS:
 *
 * RETURNS:
 *
 * EXTERNALLY READ:
 *
 * EXTERNALLY MODIFIED:
 *
 * DESCRIPTION:
 *	returns index for new Horizontal line object
 */

int 
new_H_index(void)
{
    int i;			/* temp index */

    for (i = 0; i < Hnobj; i++) {
        if (Hobjary[i].type == OBJ_NUL) break;
    }
    if (i == Hnobj) {
        Hnobj++;
        if (Hnobj > MAX_LINE) {
            fprintf(stderr, "Horiz Line array is filled\n");
            exit(1);
        }
    }
    Hobjary[i].rlist = newRectList(0);
    return i;
}

/***********************************************************************
 *
 * FUNCTION:
 *      new_pline()
 *
 * INPUTS:
 *
 * OUTPUTS:
 *
 * RETURNS:
 *	i		(int) - index for pline
 *
 * EXTERNALLY READ:
 *
 * EXTERNALLY MODIFIED:
 *
 * DESCRIPTION:
 *      returns the next available pline index
 */

int 
new_pline(void)
{
    int i;
  
    for (i = 0; i < npln; i++) {
	/*
	 * if both objects are 0 the pline is empty
	 */
        if ((plnary[i].obj1 == 0) && (plnary[i].obj2 == 0)) break;
    }
    if (i == npln) {
        npln++;
        if (npln > MAX_PLN ) {
            Warning("Pline array is filled");
	    npln--;
	    return -1;
        }
    }
    plnary[i].rlist = newRectList(0);
    return i;
}

/***********************************************************************
 *
 * FUNCTION:
 *	rem_H_object()
 *
 * INPUTS:
 *	i	(int)
 *
 * OUTPUTS:
 *
 * RETURNS:
 *
 * EXTERNALLY READ:
 *
 * EXTERNALLY MODIFIED:
 *
 * DESCRIPTION:
 *	remove horizontal line object i
 */

void 
rem_H_object(int i)
{
    Hobjary[i].type = OBJ_NUL;
    Hobjary[i].opsptr = &nul_ops;
    Hobjary[i].rlist = NULL; 
    Hobjary[i].z.hlnptr = (struct hln *)NULL;
}

/***********************************************************************
 *
 * FUNCTION:
 *	rem_object()
 *
 * INPUTS:
 *	objptr	(struct obj *)
 *
 * OUTPUTS:
 *
 * RETURNS:
 *
 * EXTERNALLY READ:
 *
 * EXTERNALLY MODIFIED:
 *
 * DESCRIPTION:
 *	removes object at objptr
 */

void
rem_object(struct obj *objptr)
{
    objptr->type = OBJ_NUL;
    objptr->opsptr = &nul_ops;
    objptr->rlist = NULL;
    objptr->z.txtptr = (struct txt *)NULL;
}

/***********************************************************************
 *
 * FUNCTION:
 *	ins_H_object()
 *
 * INPUTS:
 *	i       (int)
 *
 * OUTPUTS:
 *
 * RETURNS:
 *
 * EXTERNALLY READ:
 *
 * EXTERNALLY MODIFIED:
 *
 * DESCRIPTION:
 *	insert horizontal line object i
 */

void 
ins_H_object(int i)
{
    Hobjary[i].type = OBJ_HLN;
    Hobjary[i].opsptr = &nul_ops;
    Hobjary[i].layer = 0;
}

/***********************************************************************
 *
 * FUNCTION:
 *	clear_all_obj()
 *
 * INPUTS:	
 *
 * OUTPUTS:
 *
 * RETURNS:
 *
 * EXTERNALLY READ:
 *
 * EXTERNALLY MODIFIED:
 *
 * DESCRIPTION:
 *	removes all objects
 */

void 
clear_all_obj(void)
{
    int i;

    (*(objbck.opsptr->rem))(BACK, OBJECT);		/* remove backup */
    for (i = 0; i < nobj; i++) {
        (*(objary[i].opsptr->rem))(i, OBJECT);          /* remove object (i) */
    }
    nobj = 0;
    lobj = 0;
    nbck = BACK;
    sbck = BACK_NONE;

    all_grid_dsp = G_SOLID;                     /* init horiz. line values */
    vgrid = G_SOLID;
    all_num_dsp = TOG_ON;
    all_line_dsp = TOG_ON;
    all_id_disp = TOG_OFF;

    auto_areset = TOG_OFF;
    auto_ereset = TOG_OFF;

    utc_bar = TOG_OFF;

    auto_renum = TOG_ON;
}

/***********************************************************************
 *
 * FUNCTION:
 *	find_object()
 *
 * INPUTS:
 *	x, y 	(int)
 *
 * OUTPUTS:
 *
 * RETURNS:
 *
 * EXTERNALLY READ:
 *
 * EXTERNALLY MODIFIED:
 *
 * DESCRIPTION:
 *	returns the index from the cursor location, checks if 
 *	location has object
 */

int 
find_object(int x, int y)
{
    int i, j;

    for (i = 0; i < nobj; i++) {
	j = (lobj + i);		/* cycle about lobj */
	if (j >= nobj) j -= nobj;
        if (pointinRectList(x, y, objary[j].rlist)) {
   	     break;
 	}
    }
    if (i == nobj) j = -1;	/* didn't find anything */
    lobj = (j + 1);		/* = 0 for no find */
    if (lobj >= nobj) lobj -= nobj;
    return j;
}

/***********************************************************************
 *
 * FUNCTION:
 *	find_H_object()
 *
 * INPUTS:
 *	x,y	(int)
 *
 * OUTPUTS:
 *
 * RETURNS:
 *
 * EXTERNALLY READ:
 *
 * EXTERNALLY MODIFIED:
 *
 * DESCRIPTION:
 *	returns the index from the cursor location, checks if
 *	object is horizontal line
 */

int 
find_H_object(int x, int y)
{
    int j;			/* temp index */

    for (j = 0; j < Hnobj; j++) {
 	if (pointinRectList(x, y, Hobjary[j].rlist)) {
    	    break;
	}
    }
    if (j == Hnobj) j = -1;
    return j;
}

/***********************************************************************
 *
 * FUNCTION:
 *      find_pline()
 *
 * INPUTS:
 *      x, y    (int)
 *
 * OUTPUTS:
 *
 * RETURNS:
 *
 * EXTERNALLY READ:
 *
 * EXTERNALLY MODIFIED:
 *
 * DESCRIPTION:
 *      returns the index from the cursor location, checks if
 *      location has pline
 */

int 
find_pline(int x, int y)
{
    int i;

    for (i = 0; i < npln ; i++) {
        if (pointinRectList(x, y, plnary[i].rlist)) {
             break;
        }
    }
    if (i == MAX_PLN) {
	i = -1;      		/* didn't find anything */
    } else if ((plnary[i].obj1 == 0) && (plnary[i].obj2 == 0)) {
	i = -1;
    }
    return i;
}

/***********************************************************************
 *
 * FUNCTION:
 *	find_object_type()
 *
 * INPUTS:
 *	type       enum typ type;
 *
 * OUTPUTS:
 *
 * RETURNS:
 *
 * EXTERNALLY READ:
 *
 * EXTERNALLY MODIFIED:
 *
 * DESCRIPTION: 
 *	Finds an object of type "type";
 *	only searches through objary -- won't find horizontal lines
 */

int 
find_object_type(enum typ type)
{
    int i;

    for (i = 0; i < nobj; i++) {
        if (type == objary[i].type)
           break;
    }
    if (i == nobj) i = -1;
    return i;
}

/***********************************************************************
 *
 * FUNCTION:
 *	store_object()
 *
 * INPUTS:
 *	i	(int)
 *	state	(enum bck)
 *	type 	(int)
 *
 * OUTPUTS:
 *
 * RETURNS:
 *
 * EXTERNALLY READ:
 *
 * EXTERNALLY MODIFIED:
 *
 * DESCRIPTION:
 *	stores last state for undo
 */

int 
store_object(int i, enum bck state, int type)
{
    switch (state) {
    case BACK_DEL:
    case BACK_MOVE:
	switch(type) {
	case OBJECT:
	    (*(objary[i].opsptr->cpy))(&objbck, &objary[i]);
	    break;
	case LINE_X:
	    (*(xlnary[i].opsptr->cpy))(&objbck, &xlnary[i]);
	    break;
  	case LINE_Y:
	    (*(ylnary[i].opsptr->cpy))(&objbck, &ylnary[i]);
	    break;
	}
	break;
    case BACK_EDIT:
	switch(type) {
	case OBJECT:
            objbck = objary[i];
            objary[i] = objtmp;
	    break;
	case LINE_X:
	    objbck = xlnary[i];
	    xlnary[i] = objtmp;
	    break;
	case LINE_Y:
	    objbck = ylnary[i];
	    ylnary[i] = objtmp;
	    break;
	}
	rem_object(&objtmp);
	break;
    case BACK_ADD:
	if (type == OBJ_CHT) {
	    i = new_index(TRUE);
	} else {
            i = new_index(FALSE);
	}
        objary[i] = objtmp;
        rem_object(&objtmp);
        break;
    case BACK_NONE:
	break;
    }
    if (type == OBJECT) {
    	nbck = (state == BACK_DEL) ? BACK : i;
    	sbck = state;
    } else {
	nbck = i;
	sbck = state;
    }
    return i;
}

/***********************************************************************
 *
 * FUNCTION:
 *	recall_object()
 *
 * INPUTS:	
 *
 * OUTPUTS:
 *
 * RETURNS:
 *
 * EXTERNALLY READ:
 *
 * EXTERNALLY MODIFIED:
 *
 * DESCRIPTION:
 *	recalls last state for undo
 */

void 
recall_object(void)
{
    if (sbck == BACK_DEL) {
	nbck = new_index(FALSE);
    }
    objtmp = objary[nbck];
    objary[nbck] = objbck;
    objbck = objtmp;
    rem_object(&objtmp);
    issue_date();
    switch (sbck) {
    case BACK_ADD:
	rem_object(&objary[nbck]);
	sbck = BACK_DEL;
	nbck = BACK;
	exposeRectList(0, objbck.rlist);
        break;
    case BACK_DEL:
	sbck = BACK_ADD;
	exposeRectList(0, objary[nbck].rlist);
  	break;
    case BACK_EDIT:
    case BACK_MOVE:
	exposeRectList(1, objbck.rlist);
 	exposeRectList(0, objary[nbck].rlist);
	break;
    case BACK_NONE:
	break;
    }
}

/***********************************************************************
 *
 * FUNCTION:
 *	init_objects()	
 *
 * INPUTS:
 *	parent		(Widget)
 *	display 	(Display *)
 *	appvalue	(XtAppContext)
 *
 * OUTPUTS:
 *
 * RETURNS:
 *
 * EXTERNALLY READ:
 *
 * EXTERNALLY MODIFIED:
 *
 * DESCRIPTION:
 *	initializes the objects
 */

void 
init_objects(Widget parent, Display *display, XtAppContext appvalue, Boolean
    appleSingle)
{
    int i;					/* temp index */

    nxln = 0;
    nyln = 0;
    nobj = 0;
    Hnobj = 0;
    lobj = 0;
    ncolor = 0;
    sbck = BACK_NONE;
    app = appvalue;

    nul_ops.edt = (edt_fun)nul_func;		/* init null obj functions */
    nul_ops.rem = (rem_fun)nul_func;
    nul_ops.dsp = (dsp_fun)nul_func;
    nul_ops.lim = (lim_fun)nul_func;
    nul_ops.loc = (loc_fun)nul_func;
    nul_ops.mov = (mov_fun)nul_func;
    nul_ops.cpy = (cpy_fun)nul_func;
    nul_ops.msg = (msg_fun)nul_func;

    objtmp.type = objbck.type = OBJ_NUL;  	/* init tmp and bck object */
    objtmp.opsptr = objbck.opsptr = &nul_ops;
    objtmp.rlist = objbck.rlist = NULL;
    objtmp.z.txtptr = objbck.z.txtptr = (struct txt *)NULL;

    all_grid_dsp = G_SOLID;			/* init horiz. line values */
    vgrid = G_SOLID;
    all_num_dsp = TOG_ON;
    all_line_dsp = TOG_ON;
    all_id_disp = TOG_OFF;

    utc_bar = TOG_OFF;

    init_internals();
    init_pline();

    init_colors();				/* init_colors must be called */
    init_markers();				/* init_markers must be called*/
    init_chart_object(parent);
    init_hline_object(parent);

    init_text_object(parent);
    init_event_object(parent);
    init_sactivity_object(parent);
    init_activity_object(parent);
    init_sequence_object(parent);
    init_image_object(parent);
    init_slip_object(parent);
    init_status_object(parent);
    init_page_object(parent);
    init_chdate_object(parent);
    init_symbol_object(parent);
    init_level_object(parent);
    init_print_dialog(parent, display, appvalue, "Print Dialog");

    /* 
     * Create the chart object 
     */
    line_init();
    init_chart();
    y_gap = (int)((y_lines_end - y_act_bottom) / pg_lines);
    y_lines_end = y_gap * pg_lines + y_act_bottom;
    hline_init();

    /*
     * Run the locate function on the initial objects
     */
    for (i = 0; i < nxln; i++) {
	(*(xlnary[i].opsptr->loc))(i, LINE_X);
    }
    for (i = 0; i < nyln; i++) {
	(*(ylnary[i].opsptr->loc))(i, LINE_Y);
    }
    for (i = 0; i < Hnobj; i++) {
	(*(Hobjary[i].opsptr->loc))(i, OBJECT);
    }
    for (i = 0; i < nobj; i++) {
        (*(objary[i].opsptr->loc))(i, OBJECT);
    }
}

/***********************************************************************
 *
 * FUNCTION:
 *	check_text()
 *
 * INPUTS:
 *	item	(Widget)
 *	txtptr	(char **)
 *
 * OUTPUTS:
 *
 * RETURNS:
 *
 * EXTERNALLY READ:
 *
 * EXTERNALLY MODIFIED:
 *
 * DESCRIPTION:
 *	checks object dialog box routines and changes of dialog
 *	box items
 */

int 
check_text(Widget item, char **txtptr)
{
    int change;                 /* true if change (return value) */
    char *nval;                 /* new value of text (?) */

    nval = (char *)XmTextGetString(item);
    if (strcmp(*txtptr, nval)) {
        *txtptr = (char *)XtRealloc(*txtptr, strlen(nval) + 1);
        strcpy(*txtptr, nval);
        change = TRUE;
    } else {
        change = FALSE;
    }
    XtFree(nval);
    return change;
}

/***********************************************************************
 *
 * FUNCTION:	check_number()
 *
 * INPUTS:	item	(Widget)
 *		numptr	(int *)
 *		lb, ub	(int)
 *
 * OUTPUTS:
 *
 * RETURNS:
 *
 * EXTERNALLY READ:
 *
 * EXTERNALLY MODIFIED:
 *
 * DESCRIPTION:	checks number against dialog box item
 */

int 
check_number(Widget item, int *numptr, int lb, int ub)
{
    char *text;			/* temporary text pointer */
    int change;                 /* true if change (return value) */
    int nval;                   /* new value of number (?) */

    text = (char *)XmTextGetString(item);
    nval = atoi(text);
    if ((nval >= lb) && (nval <= ub)) {
        if (*numptr != nval) {
            *numptr = nval;
            change = TRUE;
        } else {
            change = FALSE;
        }
    } else {
        change = ERROR;
	Warning("Improper parameter");
    }    
    XtFree(text);
    return change;
}

/***********************************************************************
 *
 * FUNCTION:
 *	check_day()
 *
 * INPUTS:
 *	month, day, year    (Widget)
 *      numptr  (int *)
 *
 * OUTPUTS:
 *
 * RETURNS:
 *
 * EXTERNALLY READ:
 *
 * EXTERNALLY MODIFIED:
 *
 * DESCRIPTION:
 *	checks day against dialog box item
 */

int 
check_day(Widget month, Widget day, Widget year, int *numptr)
{
    char *text, *text3;
    int change;
    int nval, nval3;
    int leap;

    text = (char *)XmTextGetString(month);
    text3 = (char *)XmTextGetString(year);
    nval = atoi(text);
    nval3 = atoi(text3);
    if ((nval == 4) || (nval == 6) || (nval == 9) || (nval == 11)) {
	change = check_number(day, numptr, 1, 30);
    } else if (nval  == 2) {
	leap = (nval3 %4);
	if (leap == 0) {
	    change = check_number(day, numptr, 1, 29);
	} else {
	    change = check_number(day, numptr, 1, 28);
	}
    } else {
	change = check_number(day, numptr, 1, 31);
    }
    return change;
}	
    
/***********************************************************************
 *
 * FUNCTION:
 *	check_list()
 *
 * INPUTS:
 *	item	(Widget)
 *	numptr	(int *)
 *	xlate	(int [])
 *
 * OUTPUTS:
 *
 * RETURNS:
 *
 * EXTERNALLY READ:
 *
 * EXTERNALLY MODIFIED:
 *
 * DESCRIPTION:
 *	checks and translates list
 */

int 
check_list(Widget item, int *numptr, int xlate[])
{
    int change = FALSE;
    int nval, count;
    int *list[10];

    if (XmListGetSelectedPos(item, list, &count)) {
	nval = *list[0] - 1;
	if (xlate != NULL) nval = xlate[nval];
	if (*numptr != nval) {
	    *numptr = nval;
	    change = TRUE;
	}
    } 
    return change;	
}

/***********************************************************************
 *
 * FUNCTION:
 *	check_optionmenu()
 *
 * INPUTS:
 *	cur_val	(int)
 *	prev_val(int *)
 *
 * OUTPUTS:
 *
 * RETURNS:
 *
 * EXTERNALLY READ:
 *
 * EXTERNALLY MODIFIED:
 *
 * DESCRIPTION:
 *	checks if the option has changed
 */

int 
check_optionmenu(int cur_val, int *prev_val)
{
    int change = FALSE;

    if ( *prev_val != cur_val) {
        *prev_val = cur_val;
        change = TRUE;
    }
    return change;
}

/***********************************************************************
 *
 * FUNCTION:
 *	check_year()
 *
 * INPUTS:
 *	item	(Widget)
 *	numptr	(int *)
 * 
 * OUTPUTS:
 *
 * RETURNS:
 *
 * EXTERNALLY READ:
 *
 * EXTERNALLY MODIFIED:
 *
 * DESCRIPTION:
 *	checks to see if a change has occured in the year field
 */

int 
check_year(Widget item, int *numptr)
{
    int change;		/* true if change (return value) */
    int nval;		/* new vale of number */
    char *text;

    text = (char *)XmTextGetString(item);
    nval = atoi(text);
    if ((nval >= 1970) && (nval < 2070)) {
	if (*numptr != nval) {
	    *numptr = nval;
	    change = TRUE;
	} else {
	    change = FALSE;
	}
    } else {
	change = ERROR;
	Warning("Improper year parameter");
    }
    return change;
}

/***********************************************************************
 *
 * FUNCTION:
 *      check_radio_box()
 *
 * INPUTS:
 *	items		- (Widget)
 *	radioptr	- (int *)
 *	max		- (int)
 *
 * OUTPUTS:
 *
 * RETURNS:
 *      FALSE           - Indicates no change
 *      TRUE            - Indicates change
 *
 * EXTERNALLY READ:
 *
 * EXTERNALLY MODIFIED:
 *
 * DESCRIPTION:
 *      This function checks to see if the selection in the raio box menu
 *      has changed.
 */

int 
check_radio_box(Widget items[], int *radioptr, int max)
{
    int change = FALSE;
    int i;

    for (i = 0; i < max; i++) {
        if (XmToggleButtonGetState(items[i])) {
            break;
        }
    }
    if (*radioptr != i) {
        *radioptr = i;
        change = TRUE;
    }
    return change;
}

/***********************************************************************
 *
 * FUNCTION:
 *      check_titletext()
 *
 * INPUTS:
 *      item            - (Widget) text widget that contains new title
 *      txtptr          - (char *) pointer to old title object
 *
 * OUTPUTS:
 *
 * RETURNS:
 *      FALSE           - Indicates no change
 *      TRUE            - Indicates change
 *
 * EXTERNALLY READ:
 *
 * EXTERNALLY MODIFIED:
 *
 * DESCRIPTION:
 *      This function checks to see if a change has occurred in
 *      the current title field.
 */

int 
check_titletext(Widget item, char *txtptr)
{
    int change;                 /* true if change (return value) */
    char *nval;                 /* new value of text (?) */

     nval = (char *)XmTextGetString(item);
     if (strcmp(txtptr, nval)) {
       strcpy(txtptr, nval);
         change = TRUE;
     } else {
         change = FALSE;
     }
     XtFree(nval);
     return change;
}

/***********************************************************************
 *
 * FUNCTION:
 *	set_text()
 *
 * INPUTS:
 *	item	(Widget)
 *	txtptr	(char *)
 *
 * OUTPUTS:
 *
 * RETURNS:
 *
 * EXTERNALLY READ:
 *
 * EXTERNALLY MODIFIED:
 *
 * DESCRIPTION:
 *	sets text item
 */

void 
set_text(Widget item, char *txtptr)
{
    XmTextSetString(item, txtptr);
}

/***********************************************************************
 *
 * FUNCTION:
 *	set_number()
 *
 * INPUTS:
 *	item	(Widget)
 *	num	(int)
 *
 * OUTPUTS:
 *
 * RETURNS:
 *
 * EXTERNALLY READ:
 *
 * EXTERNALLY MODIFIED:
 *
 * DESCRIPTION:
 *	enables number as text item
 */

void 
set_number(Widget item, int num)
{
    char buf[8];

    (void)sprintf(buf, "%d", num);
    XmTextSetString(item, buf);
}

/***********************************************************************
 *
 * FUNCTION:
 *	set_list()
 *
 * INPUTS:
 *	item	(Widget)
 *	num	(int)
 *	xlate	(int [])
 *
 * OUTPUTS:
 *
 * RETURNS:
 *
 * EXTERNALLY READ:
 *
 * EXTERNALLY MODIFIED:
 *
 * DESCRIPTION:
 *	sets item in list widget
 */

void 
set_list(Widget item, int num, int xlate[])
{
    int i;			/* temp index */

    if (xlate != NULL) {
        i = 0;
        while (num != xlate[i]) i++;
        num = i;
    }
    XmListSelectPos(item, num+1, FALSE);
}

/***********************************************************************
 *
 * FUNCTION:
 *	set_optionmenu()
 *
 * INPUTS:
 * 	optionmenu	(Widget)
 *	itemarry	(Widget [])
 *	item	(int)
 *	xlate	(int [])
 *
 * OUTPUTS:
 *
 * RETURNS:
 *
 * EXTERNALLY READ:
 *
 * EXTERNALLY MODIFIED:
 *
 * DESCRIPTION:
 *	sets the correct item in the optionmenu
 */

void 
set_optionmenu(Widget optionmenu, Widget itemarry[], int item, int xlate[])
{
    Arg args[10];
    Cardinal n;
    int num;

    if (xlate != NULL) {
        n = 0;
        while (item != xlate[n]) n++;
        num = n;
    } else {
	num = item;
    }

    n = 0;
    XtSetArg(args[n], XmNmenuHistory, itemarry[num]); n++;
    XtSetValues(optionmenu, args, n);

}

/***********************************************************************
 *
 * FUNCTION:
 *      set_radio_box()
 *
 * INPUTS:
 *      item	- (Widget) toggle button widget to set
 *
 * OUTPUTS:
 *
 * RETURNS:
 *
 * EXTERNALLY READ:
 *
 * EXTERNALLY MODIFIED:
 *
 * DESCRIPTION:
 *      Sets the radio box widget, item, to display the number contained in
 *      num.
 */

void 
set_radio_box(Widget item)
{
    XmToggleButtonSetState(item, True, True);
}

/***********************************************************************
 *
 * FUNCTION:
 *      EditHObject()
 *
 * INPUTS:
 *      co      (int)
 *      box     (Widget)
 *
 * OUTPUTS:
 *
 * RETURNS:
 *
 * EXTERNALLY READ:
 *
 * EXTERNALLY MODIFIED:
 *
 * DESCRIPTION:
 *      brings up edit dialog box for horizontal lines
 */

void
EditHObject(int co, Widget box)
{
    struct obj *Hobjptr;		/* object pointer */

    DialogManage(box);

    if (hobject_change == TRUE) {
        Hobjary[co] = objtmp;
        rem_object(&objtmp);
        issue_date();
        modified = TRUE;

        /*
         * backup file every period
         */
        if (bck_count == 0) {
            save_file("BACKUP.xopps");
        }
        bck_count = (bck_count + 1) % PERIOD;

        exposeRectList(1, Hobjary[co].rlist);
        (*(Hobjary[co].opsptr->loc))(co, OBJECT);
        exposeRectList(0, Hobjary[co].rlist);
    } else {
        Hobjptr = &objtmp;
        rem_object(&objtmp);
        freeRectList(Hobjptr->rlist);
        XtFree((char *)Hobjptr->z.hlnptr);
    }
}

/***********************************************************************
 *
 * FUNCTION:
 *	EditObject()
 *
 * INPUTS:
 *	co	(int)
 *	box	(Widget)
 *
 * OUTPUTS:
 *
 * RETURNS:
 *
 * EXTERNALLY READ:
 *
 * EXTERNALLY MODIFIED:
 *
 * DESCRIPTION:
 *	brings up edit dialog box 
 */

void 
EditObject(int co, Widget box)
{
    int i;

    DialogManage(box);

    if (co == -1) {                             /* if new item */
        if (object_change != QUIT) {            /* done */
            if(objary[co].type == OBJ_CHT) {
                co = store_object(co, BACK_ADD, OBJECT);
                (*(objary[co].opsptr->loc))(co, OBJECT);
                issue_date();
                modified = TRUE;
                exposeRectList(0, objary[co].rlist);
            } else {
                co = store_object(co, BACK_ADD, OBJECT);
                (*(objary[co].opsptr->loc))(co, OBJECT);
                for (i = 0; i < npln; i++) {
                    loc_pline(i, OBJECT);
                }
                issue_date();
                modified = TRUE;
                exposeRectList(0, objary[co].rlist);
            }
        } else {                                /* cancel */
            (*(objtmp.opsptr->rem))(TEMP, OBJECT);
        }
    } else {                                    /* else existing item */
        if (object_change == TRUE) {            /* done */
            if (objary[co].type == OBJ_CHT) {
                /*
                 * if chart color changes, change hline color also
                 */
                for (i = 0; i < Hnobj; i++) {
                    Hobjary[i].z.hlnptr->color = objtmp.z.chtptr->color;
                    (*(Hobjary[i].opsptr->loc))(i, OBJECT);
                }
            }
            store_object(co, BACK_EDIT, OBJECT);
            issue_date();
            modified = TRUE;

            /*
             * file backup every PERIOD
             */
            if ((object_change == TRUE) && (bck_count == 0)) {
                save_file("BACKUP.xopps");
            }
            bck_count = (bck_count + 1) % PERIOD;
            exposeRectList(1, objary[0].rlist);
            (*(objary[co].opsptr->loc))(co, OBJECT);
            for (i = 0; i < npln; i++) {
                loc_pline(i, OBJECT);
                dsp_pline(i);
            }
            if (objary[co].type == OBJ_CHT) {
                for (i = 1; i < nobj; i++) {
                    (*(objary[i].opsptr->loc))(i, OBJECT);
                }
            }
            exposeRectList(1, objary[0].rlist);
        } else {                                /* cancel */
            /*
             * don't remove status object if st_flag is
             * TOG_ON and you don't want to change
             * st_flag
             */
            if ((objary[co].type != OBJ_STT) && (st_flag == TOG_OFF)) {
                (*(objtmp.opsptr->rem))(TEMP, OBJECT);
            }
        }
    }
}
